@@ -10,7 +10,7 @@ from microapp.base import MicroappObject, microapp_builtins
 from microapp.error import TestError, UsageError, InternalError
 from microapp.parse import GroupArgParser, ArgType
 from microapp.framework import load_appclass
-from microapp.utils import reduce_arg, assert_check, appdict
+from microapp.utils import reduce_arg, assert_check, appdict, Logger, Logger
 from microapp.parallel import EdgeMultiprocChildProxy as EdgeProxy
 
 PY3 = sys.version_info >= (3, 0)
@@ -55,10 +55,14 @@ class DynamicNode(Node):
 
 class AppEdge(Edge):
 
-    def append_app(self, app, args, subargs=[]):
+    def append_app(self, app, args, subargs=None):
+
         self.append_task((app, args, subargs))
 
-    def run(self, mgr, fwds=appdict()):
+    def run(self, mgr, fwds=None):
+
+        if not fwds:
+            fwds = appdict()
 
         for appname, args, subargs in self._tasks:
 
@@ -196,8 +200,10 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
         #obj.add_argument('--version', action='version', version=(cls._name_
         #        + " " + cls._version_))
 
-        if sys.version_info >= (3, 0):
-            obj.add_argument("--multiproc", help="# of processes")
+        #if sys.version_info >= (3, 0):
+        obj.add_argument("--multiproc", help="# of processes")
+
+        obj.logger = Logger(mgr, [obj._name_])
 
         return obj
 
@@ -241,7 +247,9 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
 
     def _init_multiproc(self, nprocs):
 
+
         nprocs = cpu_count() if nprocs=="*" else int(nprocs)
+        self.logger.debug("nprocs = %d" % nprocs)
 
         for idx in range(nprocs):
             self._edgeproxy_ready.append(EdgeProxy(self.manager))
@@ -263,6 +271,8 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
             child_perform, args, kwargs = proxy.pack_launch(
                                             dep.edge_forward(edge), edge, arr)
 
+            self.logger.debug("edge %d is launched" % edge._eid)
+
             # async launch process
             # TODO: evaluate using callbacks
             async_res = self._procpool.apply_async(child_perform, args, kwargs)
@@ -276,6 +286,8 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
             self._procpool.join()
 
     def run(self, args, sargs, fwds):
+    
+        self.logger.debug("Group '%s' runs: %s" % (self._name_, str(args)))
 
         # prepare env
         self._env.update(fwds)
@@ -310,9 +322,13 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
         # build graph
         if rargs and sargs:
             sargs = rargs + ["--"] + sargs
+
         elif rargs:
             sargs = rargs
 
+        elif not sargs:
+            sargs = []
+
         ret = self.connect(args, sargs)
 
         # collect entry nodes
@@ -349,11 +365,19 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
         just_arrived = []
         terminated = appdict()
 
-        if PY3 and args.multiproc:
-            self._init_multiproc(args.multiproc["_"])
+        if args.multiproc:
+            if PY3:
+                self._init_multiproc(args.multiproc["_"])
+
+            else:
+                pass
+                # TODO: show info that py2 does not support multiproc
 
         while ready_paths or self._edgeproxy_active or just_arrived:
 
+            #self.logger.debug("edgeproxy_active: %s" % str(self._edgeproxy_active))
+            #self.logger.debug("just_arrived: %s" % str(just_arrived))
+
             # launch path if exists
             if ready_paths:
                 if PY3 and args.multiproc:
@@ -383,7 +407,10 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
                     finished = proxy.process_child_message()
 
                     if finished:
+
                         arrival, edge, out, fwd = finished
+                        self.logger.debug("finished edge: %d" % edge._eid)
+
                         just_arrived.append(arrival)
 
                         if arrival in arrived:
@@ -459,6 +486,8 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
                 else:
                     raise InternalError("assert check argument is not ArgType")
 
+        self.logger.debug("Group '%s' finished." % self._name_)
+
         return 0, grp_fwds
 
 
@@ -470,7 +499,7 @@ class GroupCmd(Group):
     def __init__(self, mgr):
 
         self.add_argument("--forwarding", type=str, default="overwrite",
-                help="group forward method(pass, block, overwrite:default, list)")
+                help="group forward method(pass, block, overwrite:default, accumulate)")
 
     def connect(self, args, subargs):
 
@@ -510,7 +539,7 @@ class GroupCmd(Group):
             for v in data.values():
                 grp_fwds.update(v)
 
-        elif fwdtype == "list":
+        elif fwdtype == "accumulate":
             for k, v in data.items():
                 if k in grp_fwds:
                     grp_fwds.append(v)
