@@ -3,7 +3,7 @@
 
 from __future__ import print_function
 
-import sys, os, shlex, multiprocessing, toml, logging
+import sys, os, shlex, multiprocessing, json, logging
 
 from microapp.base import MicroappObject, microapp_builtins
 from microapp.error import (UsageError, InternalError, TypeCheckError,
@@ -75,7 +75,7 @@ class Project(MicroappObject, ProjectArgParser, Manager, DataRegistry):
         if os.path.isfile(self._cfgfile):
             with open(self._cfgfile) as f:
                 #config = json.load(f)
-                config = toml.load(f)
+                config = json.load(f)
 
         else:
             import datetime
@@ -88,7 +88,7 @@ class Project(MicroappObject, ProjectArgParser, Manager, DataRegistry):
                               "app": appdict()})
 
             with open(self._cfgfile, "w") as f:
-                toml.dump(config, f)
+                json.dump(config, f, indent=4, sort_keys=True)
 
         return config
 
@@ -179,9 +179,8 @@ class Project(MicroappObject, ProjectArgParser, Manager, DataRegistry):
     def perform(self, aargs):
         pass
 
-    def run_command(self, args=None, cwd=None, project_args=[], group_args=[],
-                app_args=[], forward=appdict(), shared=appdict(),
-                downcast=appdict()):
+    def run_command(self, args=None, cwd=None, project_args=None, group_args=None,
+                app_args=None, forward=None, shared=None, downcast=None):
         """MicroappProject command execution entry"""
 
         multiprocessing.freeze_support()
@@ -194,25 +193,41 @@ class Project(MicroappObject, ProjectArgParser, Manager, DataRegistry):
             elif isinstance(args, (str, bytes, bytearray)):
                 if sys.platform == "win32":
                     args = shlex.split(tostr(args).replace("\\", "/"))
+
                 else:
                     args = shlex.split(tostr(args))
 
             else:
                 args = [a for a in tostr_iter(args)]
 
-            project_args = [a for a in tostr_iter(project_args)]
+            if project_args:
+                project_args = [a for a in tostr_iter(project_args)]
+
+            else:
+                project_args = []
 
             if not sys.stdin.isatty():
                 project_args.append(tostr(sys.stdin.read().strip()))
 
-            group_args = [a for a in tostr_iter(group_args)]
-            app_args = [a for a in tostr_iter(app_args)]
+            if group_args:
+                group_args = [a for a in tostr_iter(group_args)]
+
+            else:
+                group_args = []
+
+            if app_args:
+                app_args = [a for a in tostr_iter(app_args)]
+
+            else:
+                app_args = []
             
         except Exception as err:
             print("ERROR: wrong argument syntax: '%s'\n" % str(args),
                     file=sys.stderr)
             sys.exit(1)
 
+        self.logger.debug("Project '%s' is started." % self._name_)
+
         try:
             ret = -1, None
 
@@ -263,8 +278,6 @@ class Project(MicroappObject, ProjectArgParser, Manager, DataRegistry):
                     handler.setFormatter(formatter)
                     self._logger.addHandler(handler)
 
-            self.logger.debug("'%s' is started." % self._name_)
-
             if sys.version_info < (3, 0) and hasattr(multiprocessing,
                     "set_start_method"):
 
@@ -278,15 +291,21 @@ class Project(MicroappObject, ProjectArgParser, Manager, DataRegistry):
 
             if pargs.forward:
                 data_transfer(pargs.forward, self._fwds)
-            self._fwds.update(forward)
+
+            if forward:
+                self._fwds.update(forward)
 
             if pargs.share:
                 data_transfer(pargs.share, self._shrds)
-            self._shrds.update(shared)
+
+            if shared:
+                self._shrds.update(shared)
 
             if pargs.downcast:
                 data_transfer(pargs.downcast, self._dcasts)
-            self._dcasts.update(downcast)
+
+            if downcast:
+                self._dcasts.update(downcast)
 
             if rargs or aargs:
 
@@ -323,22 +342,37 @@ class Project(MicroappObject, ProjectArgParser, Manager, DataRegistry):
             os.chdir(pwd)
 
             with open(self._cfgfile, "w") as f:
-                toml.dump(self._config, f)
+                json.dump(self._config, f, indent=4, sort_keys=True)
 
-        self.logger.debug("'%s' is finished." % self._name_)
+        self.logger.debug("Project '%s' is finished." % self._name_)
 
         return ret
 
-    def run_class(self, cls, cwd=None, project_args=[], group_args=[],
-                app_args=[], forward=appdict(), shared=appdict(),
-                downcast=appdict()):
+    def run_class(self, cls, cwd=None, project_args=None, group_args=None,
+                app_args=None, forward=None, shared=None, downcast=None):
         """MicroappProject app execution entry"""
 
         multiprocessing.freeze_support()
 
-        project_args = [a for a in tostr_iter(project_args)]
-        group_args = [a for a in tostr_iter(group_args)]
-        app_args = [a for a in tostr_iter(app_args)]
+        if project_args:
+            project_args = [a for a in tostr_iter(project_args)]
+
+        else:
+            project_args = []
+
+        if group_args:
+            group_args = [a for a in tostr_iter(group_args)]
+
+        else:
+            group_args = []
+
+        if app_args:
+            app_args = [a for a in tostr_iter(app_args)]
+
+        else:
+            app_args = []
+
+        self.logger.debug("Project '%s' is started." % self._name_)
 
         try:
             ret = -1, None
@@ -349,6 +383,7 @@ class Project(MicroappObject, ProjectArgParser, Manager, DataRegistry):
 
             pargs, _ = self.parse_known_args(project_args, self._env)
 
+
             if sys.version_info < (3, 0) and hasattr(multiprocessing,
                     "set_start_method"):
                 if pargs.multiproc_method:
@@ -362,15 +397,21 @@ class Project(MicroappObject, ProjectArgParser, Manager, DataRegistry):
 
             if pargs.forward:
                 data_transfer(pargs.forward, self._fwds)
-            self._fwds.update(forward)
+
+            if forward:
+                self._fwds.update(forward)
 
             if pargs.share:
                 data_transfer(pargs.share, self._shrds)
-            self._shrds.update(shared)
+
+            if shared:
+                self._shrds.update(shared)
 
             if pargs.downcast:
                 data_transfer(pargs.downcast, self._dcasts)
-            self._dcasts.update(downcast)
+
+            if downcast:
+                self._dcasts.update(downcast)
 
             if cls:
                 if issubclass(cls, Group):
@@ -409,7 +450,9 @@ class Project(MicroappObject, ProjectArgParser, Manager, DataRegistry):
             os.chdir(pwd)
 
             with open(self._cfgfile, "w") as f:
-               toml.dump(self._config, f)
+                json.dump(self._config, f, indent=4, sort_keys=True)
+
+        self.logger.debug("Project '%s' is started." % self._name_)
 
         return ret
 
@@ -433,7 +476,7 @@ class MicroappProject(Project):
     """Microapp default project"""
 
     _name_ = "microapp"
-    _version_ = "0.3.5"
+    _version_ = "0.3.7"
     _description_ = "A command-line portal to Microapp apps."
     _long_description_ = "A command-line portal to Microapp apps."
     _author_ = "Youngsung Kim"
