@@ -93,7 +93,7 @@ class ShellApp(App):
         cwd = args.workdir["_"] if args.workdir else None
         
         #import pdb; pdb.set_trace()
-        proc = Popen(cmds, stdin=PIPE, stdout=PIPE, stderr=PIPE, shell=args.useenv, cwd=cwd)
+        proc = Popen(cmds, stdout=PIPE, stderr=PIPE, shell=args.useenv, cwd=cwd)
 
         outs, errs = proc.communicate()
 
